#include "nci_api1.h"

// NCI installation mode defines
#define NCI_PRIMARYINSTALL          (0x00000001)
#define NCI_SECONDARYINSTALL        (0x00000002)

// Netcfg.dll entry point prototypes
typedef DWORD (WINAPI* NcBindings)(HWND,DWORD);
typedef DWORD (WINAPI* NcRemove)(HWND,PCWSTR,DWORD,PDWORD);
typedef DWORD (WINAPI* NcInstall)                                \
                (HWND,PCWSTR,PCWSTR,PCWSTR,PCWSTR,DWORD,PDWORD);

// Global pointers to netcfg.dll component entrypoints 
NcBindings    g_Bindings =  NULL;
NcInstall    g_Install = NULL;
NcRemove    g_Remove = NULL;

// Netcfg.dll entry point names
char g_szBindings[]    = "NetSetupReviewBindings";
char g_szInstall[]    = "NetSetupComponentInstall";
char g_szRemove[]    = "NetSetupComponentRemove";

char g_szMsg[] = "You now need to reboot the system";
HINSTANCE g_hInst = NULL;    // instance handle to netcfg.dll

// #include "nci_api2.c"  // Implementation for NCI API: Part 2

BOOL NciBindings(HWND hWnd)
{
    DWORD dwStatus = ERROR_GEN_FAILURE;
    if(g_Bindings) {

       dwStatus = g_Bindings(
                       hWnd,       // parent window
                       0           // dwBindFlags
                       );
    }
    return (dwStatus == ERROR_SUCCESS) ? TRUE : FALSE;
}    // NciBindings


BOOL NciClose()
{
    if(g_hInst != NULL) {
        g_Install = NULL;
        g_Remove = NULL;    
        g_Bindings = NULL; 
        return FreeLibrary(g_hInst);
    }
    return FALSE;

}    // NciClose

BOOL NciOpen()
{
    BOOL b = FALSE;
    // load the netcfg.dll into memory
    g_hInst = LoadLibrary("netcfg.dll");
    if(g_hInst == NULL) return b;

    // get the entrypoint to Install
    g_Install = (NcInstall) GetProcAddress(g_hInst,g_szInstall);
    g_Remove = (NcRemove) GetProcAddress(g_hInst,g_szRemove);
    g_Bindings = (NcBindings) GetProcAddress(g_hInst,g_szBindings);

    if(g_Bindings && g_Install && g_Remove && g_Bindings) b = TRUE;
    if(b == TRUE) return b;

    // Error
    NciClose();
    return b;
}    // NciOpen


BOOL NciInstall( HWND hWnd,     PWCHAR pszOption, PWCHAR pszName, 
                 PWCHAR pszPath,PWCHAR pszSym,    PDWORD pdwReturn)
{
    DWORD dwStatus = ERROR_GEN_FAILURE;
    if(g_Install) {
        dwStatus = g_Install(
                     hWnd,       // parent window
                     pszOption,     // name of the component
                     pszName,    // name of inf file
                     pszPath,            
                     pszSym,
                     NCI_SECONDARYINSTALL,
                     pdwReturn
                     );

    }
    return (dwStatus == ERROR_SUCCESS) ? TRUE : FALSE;
}    // NciInstall

BOOL NciProcessReturn(HWND hWnd, DWORD dwReturn)
{
    BOOL bResult = TRUE;
    switch(dwReturn) {
        case NCI_STATUS_SUCCESSFUL:
            bResult = NciBindings(hWnd);
            bResult &= NciReboot(hWnd);
            break;

        case NCI_STATUS_USER_CANCEL:
        case NCI_STATUS_FAILED:
        case NCI_STATUS_NO_EFFECT:
            bResult = TRUE;
            break;

        case NCI_STATUS_REBIND:
            bResult = NciBindings(hWnd);
            break;

        case NCI_STATUS_REBOOT:
            bResult = NciReboot(hWnd);
            break;

        default:
            break;
    }
    return bResult;
}    // NciProcessReturn

BOOL NciReboot(HWND hWnd)
{
    MessageBox(hWnd, g_szMsg, "Reboot", MB_OK);
    return TRUE;
}

BOOL NciRemove(HWND hWnd, WCHAR* wcInfOption, PDWORD pdwReturn)
{
    DWORD dwStatus = ERROR_GEN_FAILURE;
    if(g_Remove) {

        dwStatus = g_Remove(
                    hWnd,
                    wcInfOption,
                    NCI_SECONDARYINSTALL,
                    pdwReturn
                    );
    }
    return (dwStatus == ERROR_SUCCESS) ? TRUE : FALSE;
}    // NciRemove




